@@ -1,15 +1,15 @@
 #!/usr/bin/perl -w
 use strict;
 ##########################################################################
-# $Id: logwatch.pl,v 1.178 2006/09/16 04:04:14 kirk Exp $
+# $Id: logwatch.pl,v 1.192 2007/05/20 00:24:20 kirk Exp $
 ##########################################################################
 # Most current version can always be found at:
 # ftp://ftp.logwatch.org/pub/redhat/RPMS
 
 ########################################################
 # Specify version and build-date:
-my $Version = '7.3.1';
-my $VDate = '09/15/06';
+my $Version = '7.3.6';
+my $VDate = '05/19/07';
 
 #######################################################
 # This was written and is maintained by:
@@ -64,7 +64,7 @@ $Config{'detail'} = 0;
 $Config{'mailto'} = "root";
 $Config{'mailfrom'} = "Logwatch";
 $Config{'save'} = "";
-$Config{'print'} = 0;
+$Config{'print'} = 1;
 $Config{'range'} = "yesterday";
 $Config{'debug'} = 0;
 $Config{'archives'} = 1;
@@ -77,15 +77,20 @@ $Config{'pathtozcat'} = "zcat";
 $Config{'pathtobzcat'} = "bzcat";
 $Config{'output'} = "unformatted";
 $Config{'encode'} = 0;
+$Config{'html_wrap'} = 80;
 
 if (-e "$ConfigDir/conf/html/header.html") {
    $Config{'html_header'} = "$ConfigDir/conf/html/header.html";
+} elsif (-e "$BaseDir/dist.conf/html/header.html") {
+   $Config{'html_header'} = "$BaseDir/dist.conf/html/header.html";
 } else {
    $Config{'html_header'} = "$BaseDir/default.conf/html/header.html";
 }
 
 if (-e "$ConfigDir/conf/html/footer.html") {
    $Config{'html_footer'} = "$ConfigDir/conf/html/footer.html";
+} elsif (-e "$BaseDir/dist.conf/html/footer.html") {
+   $Config{'html_footer'} = "$BaseDir/dist.conf/html/footer.html";
 } else {
    $Config{'html_footer'} = "$BaseDir/default.conf/html/footer.html";
 }
@@ -268,9 +273,6 @@ if ($Config{'debug'} > 8) {
 
 CleanVars();
 
-my $OldMailTo = $Config{'mailto'};
-my $OldPrint  = $Config{'print'};
-
 # For each of the configuration sets (logwatch.conf here, and
 # logfiles,and services later), we do the following:
 #  1. read the different configuration files
@@ -307,12 +309,6 @@ for (my $i = 0; $i <= $#ReadConfigNames; $i++) {
 
 CleanVars();
 
-if ($OldMailTo ne $Config{'mailto'}) {
-   $Config{'print'} = 0;
-} elsif ($OldPrint ne $Config{'print'}) {
-   $Config{'mailto'} = "";
-}
-
 if ($Config{'debug'} > 8) {
    print "\nConfig After Config File:\n";
    PrintConfig();
@@ -324,16 +320,14 @@ my @TempLogFileList = ();
 my @TempServiceList = ();
 my $Help = 0;
 my $ShowVersion = 0;
-
-$OldMailTo = $Config{'mailto'};
-$OldPrint  = $Config{'print'};
+my $tmp_mailto;
 
 GetOptions ( "d|detail=s"   => \$Config{'detail'},
              "l|logfile=s@" => \@TempLogFileList,
              "logdir=s"     => \$Config{'logdir'},
              "s|service=s@" => \@TempServiceList,
              "p|print"      => \$Config{'print'},
-             "m|mailto=s"   => \$Config{'mailto'},
+             "m|mailto=s"   => \$tmp_mailto,
              "save=s"       => \$Config{'save'},
              "a|archives"   => \$Config{'archives'},
              "debug=s"      => \$Config{'debug'},
@@ -345,7 +339,8 @@ GetOptions ( "d|detail=s"   => \$Config{'detail'},
              "splithosts"   => \$Config{'splithosts'},
              "multiemail"   => \$Config{'multiemail'},
              "o|output=s"   => \$Config{'output'},
-             "encode"       => \$Config{'encode'}
+             "encode"       => \$Config{'encode'},
+             "html_wrap=s"  => \$Config{'html_wrap'}
            ) or Usage();
 
 $Help and Usage();
@@ -360,6 +355,11 @@ if ($ShowVersion) {
    exit 0;
 }
 
+if ($tmp_mailto) {
+   $Config{'mailto'} = $tmp_mailto;
+   $Config{'print'} = 0;
+}
+
 CleanVars();
 
 
@@ -405,10 +405,8 @@ if (($Config{'save'} ne "") && (-e "$Config{'save'}") ) {
    unlink "$Config{'save'}";
 }
 
-if ($OldMailTo ne $Config{'mailto'}) {
-   $Config{'print'} = 0;
-} elsif ($OldPrint ne $Config{'print'}) {
-   $Config{'mailto'} = "";
+if ($Config{'mailto'} eq "") {
+   $Config{'print'} = 1;
 }
 
 if ($Config{'debug'} > 8) {
@@ -896,7 +894,7 @@ foreach $LogFile (@LogFileList) {
 
    foreach my $ThisFile (@FileList) {
       #Existence check for files -mgt
-      next unless (-e $ThisFile);
+      next unless (-f $ThisFile);
       if (! -r $ThisFile) {
          print "File $ThisFile is not readable.  Check permissions.";
          if ($> != 0) {
@@ -1017,11 +1015,15 @@ sub initprint {
          #In theory this should be able to handle many different mailers. I might need to add
          #some filter code on $Config{'mailer'} to make it more robust. -mgt
          open(OUTFILE,"|$Config{'mailer'}") or die "Can't execute $Config{'mailer'}: $!\n";
-         print OUTFILE "To: $Config{'mailto'}\n";
+         my $mailto = $Config{"mailto_$Config{'hostname'}"};
+         $mailto = $Config{'mailto'} unless $mailto;
+         for my $to (split(/ /, $mailto)) {
+            print OUTFILE "To: $to\n";
+         }
          print OUTFILE "From: $Config{'mailfrom'}\n";
          print OUTFILE "Subject: Logwatch for $Config{'hostname'} (${OStitle})\n";
          #Add MIME
-         $out_mime .= "MIME-Version: 1.0\n"; 
+         $out_mime = "MIME-Version: 1.0\n"; 
          if ( $Config{encode} == 1 ) {
             $out_mime .= "Content-transfer-encoding: base64\n";
          } else {
@@ -1262,9 +1264,11 @@ sub parselogs {
       } 
 
       #Set up out_reference
+      output("ul","<a name=top><ul>", "ref_extra") if defined( $index_par );
       foreach ( 0 .. $index_par ) {
          output($_,$reports[$_], "ref") if defined( $reports[$_] );
       }
+      output("ul","</ul></a>", "ref_extra") if defined( $index_par );
 
    }
    
@@ -1299,6 +1303,11 @@ sub parselogs {
 #ends here
 
    if ($Config{'multiemail'} eq 1) {
+      $out_foot = '';
+      $out_head = '';
+      $out_mime = '';
+      $out_reference = '';
+      @reports = ();
       close(OUTFILE) unless ($Config{'print'} eq 1);
    }
 }
@@ -1324,6 +1333,10 @@ sub output {
    my ($index, $text, $type) = @_;
    #Types are start stop header line ref
 
+   if ( $type eq "ref_extra" ) {
+      $out_reference .= "$text\n";
+   }
+
    if ( $type eq "ref" ) {
       $out_reference .= "   <li><a href=\"#$index\">$reports[$index]</a>\n";
    }
@@ -1334,15 +1347,15 @@ sub output {
       if ( $outtype_html ) {
          $out_body{$index} .= " <hr>
          <h2><a name=\"$index\">$reports[$index]</a></h2>
-         <div class=\"sample\">
-         <table border=\"1\">\n";
+         <div class=service>
+         <table border=1 width=100%>\n";
       }
    } 
 
    if ( $type eq "stop" ) {
       if ( $outtype_html ) {
          $out_body{$index} .= "  </table></div>\n";
-         $out_body{$index} .= "  <div class=\"explan\"><p><a href=\"#top\">Back to Top</a></p></div>\n";
+         $out_body{$index} .= "  <div class=return_link><p><a href=\"#top\">Back to Top</a></p></div>\n";
       }
    }
 
@@ -1350,10 +1363,16 @@ sub output {
 	   if ($outtype_unformatted) {
 	      $out_body{$index} .= "$text \n";
       } elsif ( $outtype_html ) {
-         $text =~ s/ +/ /go;
-         $text =~ s/(^ | $)//go;
+         #Covert spaces
+         $text =~ s/  / \&nbsp;/go;
+         #Covert tabs 1 to 4 ratio
+         $text =~ s/\t/ \&nbsp\;\&nbsp\;\&nbsp\;\&nbsp\;/go;
+         #Filters
+         $text =~ s/ $//go;
          $text = '&nbsp;' if ( $text eq '' );
-         $out_body{$index} .= "<tr>\n    <th>$text</th>   </tr>\n";
+         #This will make sure no unbroken string is longer then x characters
+         $text =~ s/(\S{$Config{html_wrap}})/$1 /g;
+         $out_body{$index} .= "<tr>\n    <th>$text</th>\n   </tr>\n";
       } else {
          $out_body{$index} .=
          sprintf( substr( $text, 0, $format[0] ) . ' ' x( $format[0] - length($text) ) . " \n" );
@@ -1364,14 +1383,21 @@ sub output {
 	   if ($outtype_unformatted or $outtype_mail) {
          $out_body{$index} .= "$text ";
       } elsif ( $outtype_html ) {
-         $text =~ s/ +/ /go;
+         #Covert spaces
+         $text =~ s/  / \&nbsp;/go;
+         #Covert tabs 1 to 4 ratio
+         $text =~ s/\t/ \&nbsp\;\&nbsp\;\&nbsp\;\&nbsp\;/go;
+         #Filters
          $text =~ s/ $//go;
          $text =~ s/</\&lt\;/go;
          $text =~ s/>/\&gt\;/go;
-         unless ( $text =~ s/^ //go ) {
-            $out_body{$index} .= "  <tr>\n    <th align=\"left\">$text</th>\n  </tr>\n";
+         #This will make sure no unbroken string is longer then x characters
+         $text =~ s/(\S{$Config{html_wrap}})/$1 /g;
+         #Grey background for spaced output
+         if ( $text =~ m/^ / ) {
+            $out_body{$index} .= "  <tr>\n    <td bgcolor=#dddddd>$text</td>\n  </tr>\n";
          } else {
-            $out_body{$index} .= "  <tr>\n    <td bgcolor=\"#dddddd\">&nbsp;&nbsp;$text</td>\n  </tr>\n";
+            $out_body{$index} .= "  <tr>\n    <th align=left>$text</th>\n  </tr>\n";
          }
       } else {
 	      if ( length($text) > $format[0] ) {
